class CalibrationsController < ApplicationController
  include MachineResponse
  include MachineDataViewFilter
  include RxModule::Tableable

  wrap_parameters :calibration, include: [:target, :target_name, :type, :subtype, :mode, :user, :measurements, :corrections, :sn]
  model_data_store(Calibration)

  def add
    begin
      AddCalibrationJob.perform_later(model_params, measurement_params, correction_params)
    rescue => e
      Rails.logger.error e
    end
    send_machine_response
  end

  def before_set_targets
    models = if filters["sn"]
      Calibration.where(machine_sn: filters["sn"])
    else
      Calibration
    end
    view_before_set_targets(models, params)
  end

  def before_filter(filter)
    view_before_filter(filter)
  end

  private

  # Only allow a list of trusted parameters through.
  def model_params
    calibration = params.require(:calibration)
    {
      target: calibration[:target],
      target_name: calibration[:target_name],
      calibration_type: calibration[:type],
      sub_type: calibration[:subtype],
      mode: calibration[:mode],
      user: calibration[:user],
      time: "#{params[:date]} #{params[:time]}",
      machine_sn: params[:sn]
    }
  end

  def measurement_params
    measurement = params.require(:calibration).permit(measurements: {measurement: [:name, :value]}).dig(:measurements)&.dig(:measurement) || []
    Array(measurement).map do |measurements|
      {
        name: measurements[:name],
        value: measurements[:value]
      }
    end
  end

  def correction_params
    correction = params.require(:calibration).permit(corrections: {correction: [:name, :value]}).dig(:corrections)&.dig(:correction) || []
    Array(correction).map do |corrections|
      {
        name: corrections[:name],
        value: corrections[:value]
      }
    end
  end
end
