class CoatingController < ApplicationController
  # POST /coating/1.0/batch/generate
  def generate
    require 'nokogiri'
    params = generate_params

    primary_sector_id = params[:sectors][:sectorId01]

    return render xml: response_error('Sector ID not provided for sector01'), status: :unprocessable_entity if primary_sector_id.blank?

    batch_run = BatchRun.includes(:batch_run_data).where(run_key: primary_sector_id).first

    return render xml: response_error("No batch run found for Sector ID #{primary_sector_id}"), status: :not_found if batch_run.nil?

    recipe_code = batch_run.batch_run_data.where(name: 'recipeCode').first
    dome_side = batch_run.batch_run_data.where(name: 'domeSide').first

    return render xml: response_error("Missing dome side or recipe for Batch Run #{primary_sector_id}"), status: :not_found if recipe_code.nil? || dome_side.nil?

    render xml: response_ok(batch_run.batch.batch_id, recipe_code.value, dome_side.value)
  end

  private

  def response_error(error)
    Nokogiri::XML::Builder.new(:encoding => 'UTF-8') do |xml|
      xml.batchGenerateResponseInvalid(nil, :batchId => 'error', :error => "#{error}")
    end
  end

  def response_ok(batch_id, recipe_barcode, dome_side)
    Nokogiri::XML::Builder.new(:encoding => 'UTF-8') do |xml|
      xml.batchGenerateResponseOk(nil, :batchId => batch_id, :recipeBarcode=> recipe_barcode, :lensside=>dome_side)
    end
  end

  # Only allow a list of trusted parameters through.
  def generate_params
    batch = params.require(:batchSectorList).permit(:sectorId01, :sectorId02, :sectorId03, :sectorId04, :sectorId05, :sectorId06)
    machine = Machine.find_or_create_by(sn: params[:sn])
    {
      sn: machine.sn,
      sectors: {
        sectorId01: batch[:sectorId01],
        sectorId02: batch[:sectorId02],
        sectorId03: batch[:sectorId03],
        sectorId04: batch[:sectorId04],
        sectorId05: batch[:sectorId05],
        sectorId06: batch[:sectorId06]
      },
      date: params[:date],
      time: params[:time]
    }
  end
end

