class MeiToMesController < ApplicationController
  include MachineResponse
  include MachineTranslator

  class InvalidMachineState < StandardError; end
  class InvalidLensSide < StandardError; end
  class InvalidJobState < StandardError; end

  def machine_status_add
    begin
      params = machine_status_params
      machine = Machine.find_or_create_by(sn: params[:machine_sn])
      model_params = MachineTranslator.get_translator(machine.translator_class)
                                      .get_translation(params, MachineTranslator.machine_state_type)
      AddMachineStateJob.perform_later(model_params)
    rescue => e
      Rails.logger.error "Error in adding Machine State from MEI - #{e}"
    end
    send_machine_response
  end

  def job_status_add
    begin
      params = job_status_params
      model_params = MachineTranslator.get_translator(params[:machine].translator_class)
                                      .get_translation(params, MachineTranslator.job_state_type)
      AddJobState.perform_later(model_params)
    rescue => e
      Rails.logger.error "Error in adding Job State from MEI - #{e}"
    end
    send_machine_response
  end

  def alarm_add
    begin
      params = alarm_params
      machine = Machine.find_or_create_by(sn: params[:machine_sn])
      model_params = MachineTranslator.get_translator(machine.translator_class)
                                      .get_translation(params, MachineTranslator.error_type)
      AddErrorJob.perform_later(model_params)
    rescue => e
      Rails.logger.error "Error in adding Alarm from MEI - #{e}"
    end
    send_machine_response
  end

  def quality_control_add
  #   Potential endpoint for quality control
  end

  private

  def machine_status_params
    machine_state = params.require(:MtmpMachineStatus)
    status = machine_state[:Status].downcase

    allowed_machine_states = %w[working workingidle on notready ready alarm off]
    raise InvalidMachineState if allowed_machine_states.exclude?(status)

    {
      state: status,
      time: "#{params[:date]} #{params[:time]}",
      machine_sn: params[:sn]
    }
  end

  def job_status_params
    job_state = params.require(:MtmpJobStatus)
    job_number = job_state[:Job]
    machine = Machine.find_or_create_by(sn: params[:sn])
    lens_side = job_state[:LensSide].downcase
    status = job_state[:Status].downcase

    allowed_lenssides = %w[r l unknown]
    allowed_job_states = %w[started completed aborted]
    raise InvalidLensSide if allowed_lenssides.exclude?(lens_side)
    raise InvalidJobState if allowed_job_states.exclude?(status)

    # Default 'unknown' lens side to left
    lens_side = "l" if lens_side == "unknown"
    side_num = Sideable.side_letter_to_side_num(lens_side)
    job = FindHelpers::JobFinders.find_active_or_create_by_number_and_side_num(job_number, side_num)

    {
      job: job,
      machine: machine,
      process: job_state[:Process],
      side: side_num,
      state: status,
      comment: job_state[:Comment]
    }
  end

  def alarm_params
    error = params.require(:MtmpAlarm)

    {
      piececounter: 0,
      errno: 0,
      error: error[:text],
      errortype: 2,
      status: 0,
      time: "#{params[:date]} #{params[:time]}",
      machine_sn: params[:sn]
    }
  end

  def quality_control_params
    # TODO: format process params here
  end
end
