class StateBreakdownDashboard < RxModule::Dashboards::CategoryChart
  include EerDashboard
  include MachineDashboard

  #TODO: Disabled until we review EER (June 8th 2022)

=begin
  include MachineConcern
  expose('state_breakdown', 'Machine State Breakdown', 'View the percentage of time spent in each EER state for the selected machine(s)')
  targets("Machine", "The machine(s) to track", -> { machine_targets }, target_type: "series")
  offers_categories(-> { categories })
  parameter(:history_length, "History", "How many much time to go back when fetching data", :timelength, default: 86400, min: 1)
  parameter(:multimode, "Machine Groups", "How to handle multiple machine selections", :string, default: 'average', value_list: [
    { label: 'Show Each Machine', value: "individual" },
    { label: 'Show Average Time In State', value: "average" },
    # { label: 'Show Total Time In State', value: "sum" },
  ])
  ui_hint(category: 'EER')
  yunit('minutes')
=end
  def combine_mode
    :average
  end

  def unroll_target?(target)
    target.is_a?(MachineGroup) && (@parameters[:multimode] == 'individual')
  end

  def build_results!
    setup_series!
    process_events!
    add_shutdown_points!
    scale_groups!
    convert_data_to_minutes!
  end

  def setup_series!
    parsed_targets.each do |target|
      if unroll_target?(target)
        target.machines.each do |sub_target|
          push_series(target_to_id(sub_target), target_to_name(sub_target)) unless get_series(target_to_id(sub_target))
        end
      else
        push_series(target_to_id(target), target_to_name(target)) unless get_series(target_to_id(target))
      end
    end
  end

  def process_events!
    eer_for_range.each do |event|
      this_range = (event.start_time)..(event.closed_time || end_time)
      time_in_event = scheduled_range_data.reduce(0) do |sum, r|
        sum + intersection_length(r, this_range)
      end
      if time_in_event > 0
        pushed_targets = [] # a machine could be included 3 ways: directly requested, included by a machine group, included to calculate an average
        parsed_targets.each do |target|
          if event_applies?(target, event)
            if unroll_target?(target)
              sub_target = target.machines.find { |m| m.id == event.id }
              unless pushed_targets.include?(sub_target)
                pushed_targets << sub_target
                increment_point(target_to_id(sub_target), time_in_event, event.eer_category)
              end
            else
              unless pushed_targets.include?(target)
                pushed_targets << target
                increment_point(target_to_id(target), time_in_event, event.eer_category||event.event_type)
              end
            end
          end
        end
      end
    end
  end

  def add_shutdown_points!
    dataset.each do |s|
      active_time = s[:data].reduce(0) do |sum, point|
        sum + point[:val]
      end
      expected_active_time = if s[:id][0] == 'm' #machine
                               total_active_time
                             else
                               machine_group = target_from_id(s[:id])
                               total_active_time * machine_group.machines.length # each machine multiplies the total available time
                             end
      if active_time < expected_active_time
        push_point(s[:id], expected_active_time - active_time, "Shutdown")
      end
    end
  end

  def scale_groups!
    if @parameters[:multimode] == 'average'
      dataset.each do |s|
        if (target = target_from_id(s[:id]))&.is_a?(MachineGroup)
          s[:data].each do |point|
            point[:val] = point[:val] / target.machines.length
          end
        end
      end
    end
  end

  def convert_data_to_minutes!
    dataset.each do |s|
      s[:data].each do |point|
        point[:val] = point[:val] / 60
      end
    end
  end

  def restrict_to_overtime_eer
    true
  end

  def start_time
    @_start_time ||= (call_time - (@parameters[:history_length] || 86400).seconds)
  end

  def end_time
    @_end_time ||= call_time
  end

  class << self
    def categories
      RxModule::Communicator.call_module_method("rx_eer", "/api/eer_categories/all_categories", :get)[:models].map do |c|
        { value: c[:name], label: c[:name], color: c[:color] }
      end + [{ value: "Shutdown", label: "Shutdown", color: "#C0C0C0" }]
    rescue
      [{ value: "Shutdown", label: "Shutdown", color: "#C0C0C0" }]
    end

    def machine_targets
      MachineGroup.all.map do |g|
        { label: target_to_name(g), value: target_to_id(g), group: "Machine Group", description: "Report on percent uptime for the entire #{g.name} group" }
      end + Machine.all.map do |m|
        { label: target_to_name(m), value: target_to_id(m), group: "Machine", description: "Report on percent uptime for machine #{m.name}, sn #{m.sn}" }
      end
    end
  end
end