import * as React from 'react';
import {Layout, Providers} from '@rx-suite/ui';
import {handleWorkstationLogin, RxAppModel, RxAppStore, RxModuleInfoStore} from '@rx-suite/stores';
import 'react-table/react-table.css';
import {observer} from "mobx-react-lite";
import {LinearProgress} from "@material-ui/core";
import { subscribe } from "@rx-suite/data";
import {t} from "@lingui/macro";
import {getRoutes} from "./routes";
import { getBlockSettings } from './stores/BlockReuseSettingStore';

subscribe();
handleWorkstationLogin();
export default observer(() => {
    const [ messages, setMessages ] = React.useState<any>(null);
    const [ blockReuseSettings, setBlockReuseSettings ] = React.useState<any>(null);
    const model = RxAppStore.selectedModel as RxAppModel;
    let version = "";
    React.useEffect(() => {
        const getMessages = async () => {
            const compiledMessages = await import(`../../../locales/${model.language}/messages.mjs`);
            setMessages(compiledMessages.messages);
        }
        const getBlockReuseSettings = async () => {
            const settings = await getBlockSettings();
            setBlockReuseSettings(settings);
        }
        if (model) {
            model.setTitleBase(t`Rx-Data`)
        }
        getMessages();
        getBlockReuseSettings();
    }, [model]);
    if(RxModuleInfoStore.selectedModel) {
        version = RxModuleInfoStore.selectedModel.version as string;
    }
    if(!model || !messages) {
        return <LinearProgress />
    }
    return(
        <Providers theme={model.theme} language={model.language} messages={messages}>
            <Layout logoSrc={"/api/rx_module/logo"} routes={getRoutes(blockReuseSettings)} version={version}/>
        </Providers>
    )
});


