import * as React from "react";
import {observer} from "mobx-react-lite";
import {Card, Grid, makeStyles} from "@material-ui/core";
import Typography from "@material-ui/core/Typography";
import {grey} from "@material-ui/core/colors";
import {BlockPickingModel} from "../../stores/BlockPickingStore";
import {t} from "@lingui/macro";
import {ShiftStore} from "../../stores/ShiftStore";

type BlockPickedCountCardProps = {
    model: BlockPickingModel
};

const useStyles = makeStyles((theme) => ({
    card: {
            display: 'flex',
            alignItems: 'center',
            width: "19%",
            marginLeft: "1%",
            height: "100%"
    },
    shiftLabel: {
        textAlign: 'center',
        color: grey[600]
    },
    textCenter: {
        textAlign: 'center'
    }
}));

export const BlockPickedCountCard: React.FunctionComponent<BlockPickedCountCardProps> = observer((props) => {

    const {model} = props;
    const classes = useStyles();

    const count = model.active_shift_id ? model.block_count : t`Not Available`;
    let shiftText = t`No Active Shift`;

    const shiftData = ShiftStore.get(model.active_shift_id);
    const getShiftName = () => {
        if (shiftData) {
            shiftText = t`Shift` + `: ${shiftData.name as string}`;
        }
    };

    if (model.hide_block_count) {
        return null;
    }

    if (model.active_shift_id) {
        getShiftName();
    }

    return (
        <Card className={classes.card}>
            <Grid container direction="column" alignItems="center">
                <Typography className={classes.textCenter} variant='h6'>{t`Blocks Picked`}</Typography>
                <Typography className={classes.textCenter} variant='h4'>{count}</Typography>
                <Typography className={classes.shiftLabel} variant='h6'>{shiftText}</Typography>
            </Grid>
        </Card>
    );
});
