import React from "react";
import {observer} from "mobx-react-lite";
import {FormProps, ModelInput, ModelProvider, ModelSelect} from "@rx-suite/ui/build";
import {
    Button,
    createStyles,
    Dialog,
    DialogActions,
    DialogContent,
    DialogTitle,
    makeStyles,
    Theme
} from "@material-ui/core";
import {Model} from "@rx-suite/data/build";
import {Trans} from "@lingui/macro";

interface BlockPickingTagDialogProps extends FormProps{
    open: boolean,
    handleSubmit: (model:Model) => void,
    handleClose: () => void
}

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        bold: {
            fontWeight: 'bold'
        },
        code: {
            fontWeight: 'bold',
            backgroundColor: 'lightgray',
            padding: '2px 5px'
        }
    })
);

export const BlockPickingTagDialogComponent: React.FunctionComponent<BlockPickingTagDialogProps> = observer(({model, open, handleSubmit, handleClose}) => {

    const classes = useStyles();
    const regexExample = ".{3}$"

    return (
        <div>
            <ModelProvider value={model}>
                <Dialog open={open} onClose={handleClose} aria-labelledby="form-dialog-title" maxWidth={"md"} fullWidth={true}>
                    <DialogTitle id="form-dialog-title">Add Capability</DialogTitle>
                    <DialogContent>
                        <ModelInput name={'name'}/>
                        <ModelInput name={'tag'}/>
                        <ModelInput name={'regex'}  hint={{title: 'Regex', children:
                                <div>
                                    <div><Trans>This Regex will be applied to the tag value returned and can be used to transform the value.</Trans></div>
                                    <br/>
                                    <div><span className={classes.bold}><Trans>Examples:</Trans></span></div>
                                    <ul>
                                        <li><span className={classes.code}>{regexExample}</span> : <Trans>Extract the last 3 characters of the tag value.</Trans></li>
                                    </ul>
                                </div>
                        }}/>
                        <DialogActions>
                            <Button onClick={handleClose} color={"secondary"}>
                                close
                            </Button>
                            <Button onClick={e => handleSubmit(model)} color={"secondary"}>
                                Submit
                            </Button>
                        </DialogActions>
                    </DialogContent>
                </Dialog>
            </ModelProvider>
        </div>
    )
});
