import * as React from "react";
import {observer} from "mobx-react-lite";
import {t} from "@lingui/macro";
import {ModelCheckbox, ModelSelect} from "@rx-suite/ui";
import {BlockPickingModel} from "../../../stores/BlockPickingStore";
import Typography from "@material-ui/core/Typography";
import {ShiftStore} from "../../../stores/ShiftStore";
import {LinearProgress} from "@material-ui/core";

type BPDataSettingsTabTabProps = {
    model: BlockPickingModel
};

export const BPDataSettingsTab: React.FunctionComponent<BPDataSettingsTabTabProps> = observer((props) => {

    const {model} = props;

    const shifts = ShiftStore.getAll();

    const shiftOptions = React.useCallback(() => {
        return shifts.map((model) => {
            return {
                label: `${model.name}`,
                value: model.getId()
            };
        });
    }, [shifts]);

    const shiftSelector = React.useCallback(() => {
        if(ShiftStore.isLoading) {
            return <LinearProgress/>
        }
        return <ModelSelect label={t`Shifts to use for Block Count`} name={'hub_shift_ids'} options={shiftOptions()} multiple={true}/>
    }, [ShiftStore.isLoading])

    return (
        <>
            <Typography variant={'h6'}>{t`Job Data`}</Typography>
            <ModelCheckbox name={'send_job_states'} label={t`Send Job States`} hint={{title: t`Send Job States`, text: t`Send a job state FINISHED for each lens after a successful scan. Note: This feature uses the Block Picking station name as the serial number.` }} />
            <Typography variant={'h6'}>{t`Block Count`}</Typography>
            {shiftSelector()}
            <ModelCheckbox name={'use_scan_count'} label={t`Use Scan Count`} hint={{title: t`Use Scan Count`, text: t`When enabled, every scan is counted as a single block piece even when DO=B.` }} />
        </>
    );
});
