import * as React from "react";
import {observer} from "mobx-react-lite";
import {t, Trans} from "@lingui/macro";
import {ModelCheckbox, ModelInput} from "@rx-suite/ui";
import {createStyles, Divider, Theme, Typography} from "@material-ui/core";
import {makeStyles} from "@material-ui/core/styles";
import {BlockPickingModel} from "../../../stores/BlockPickingStore";

type LmsLegacySettingsProps = {
    model: BlockPickingModel
}

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        bold: {
            fontWeight: 'bold'
        }
    })
);

export const LmsLegacySettings: React.FunctionComponent<LmsLegacySettingsProps> = observer(({model}) => {
    const classes = useStyles();

    if(!model.isUsingLegacyConnection || model.isNew){
        return null
    }

    return (
        <React.Fragment>
            <ModelInput disabled label={t`LMS IP Address`} name={'ip_address'}/>
            <ModelInput disabled label={t`Port`} name={'port'}/>
            <ModelInput disabled label={t`Device Type`} name={'device_type'} hint={{title: 'Device Type', children:
                    <div>
                        <div><Trans>Value of the Device Type (DEV) field in the LMS initializer.</Trans></div>
                        <br/>
                        <div><Trans><span className={classes.bold}>Recommended Value: BLK</span></Trans></div>
                    </div>
            }}/>
            <ModelInput disabled label={t`Delay on Socket Open (seconds)`} name={'delay_on_socket_open'} hint={{title: "Delay on Socket Open", text: "When greater than 0, the system will wait at least X seconds before sending the first INI message after opening the TCP/IP socket."}}/>
            <ModelCheckbox disabled name={'hold_connection'} label={'Hold Connection Open'}/>
        </React.Fragment>
    )
});
