import * as React from "react";
import {observer} from "mobx-react-lite";
import {Column, Filter, SortingRule} from "react-table";
import {MachineModel} from "../../stores/MachineStore";
import {PeriodFilterButtons} from "./PeriodFilterButtons";
import {DataStore, Model} from "@rx-suite/data";
import {Component} from "react";
import {makeStyles} from "@material-ui/core";
import {MachineTypeStore} from "../../stores/MachineTypeStore";
import {NoCollectMachineTypeInfo} from "./NoCollectMachineTypeInfo";
import {MachineDataViewTable} from "./MachineDataViewTable";
import {MachineDataViewExport} from "./MachineDataViewExport";

type MachineDataViewTabProps = {
    machine: MachineModel;
    enabledKey: string;
    tabName: string;
    columns: Column[];
    store: DataStore<Model>;
    subComponent?: (original: Model) => Component | undefined;
    defaultSorted?: SortingRule;
    defaultFiltered?: Filter[];
    exportedFileName?: string;
    exportEndpoint?: string;
}

const useStyles = makeStyles((theme) => ({
    buttonRow: {
        display: 'flex',
        flexDirection: 'row',
        justifyContent: 'space-between',
        alignItems: 'flex-start'
    }
}));

export const MachineDataViewTab: React.FunctionComponent<MachineDataViewTabProps> = observer((props) => {
    const {machine, columns, store, subComponent, defaultSorted, defaultFiltered, exportedFileName, exportEndpoint, enabledKey, tabName} = props;

    const classes = useStyles();

    const types = MachineTypeStore.getAll();
    const machineType = React.useMemo(() => {
        return MachineTypeStore.models.find(type => type.id == machine.machine_type_id)
    }, [MachineTypeStore.models]);

    const [periodFilterNum, setPeriodFilterNum] = React.useState<number>(0);

    if (machineType && !machineType[enabledKey]) {
        return <NoCollectMachineTypeInfo infoType={tabName} />;
    }

    return (<>
        <div className={classes.buttonRow}>
            <PeriodFilterButtons value={periodFilterNum} change={setPeriodFilterNum} />
            <MachineDataViewExport
                machine={machine}
                store={store}
                exportedFileName={exportedFileName}
                exportEndpoint={exportEndpoint}
            />
        </div>
        <MachineDataViewTable
            machine={machine}
            store={store}
            columns={columns}
            periodFilterNum={periodFilterNum}
            subComponent={subComponent}
            defaultSorted={defaultSorted}
            defaultFiltered={defaultFiltered}
        />
    </>);
});
