import * as React from 'react';
import {makeColumn, ModelTable,} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import {ErrorFilterModel, ErrorFilterStore} from "../../stores/ErrorFilterStore";
import {observer} from "mobx-react-lite";
import {MachineTypeModel} from "../../stores/MachineTypeStore";
import {ToggleFilterSwitch} from "../generic/ToggleFilterSwitch";

type MachineTypeErrorFilterListProps = {
    machineType: MachineTypeModel
}

export const MachineTypeErrorFilterList : React.FunctionComponent<MachineTypeErrorFilterListProps> = observer(({machineType}) => {
    const columns = React.useMemo(() => {
        return [
            makeColumn('name', t`Rule Name`, "string"),
            makeColumn('error_codes', t`Error Codes`, "string"),
            makeColumn('behaviour', t`Behaviour`, "string"),
            {
                Header: t`Enabled`,
                accessor: d => d,
                id: 'enabled',
                Cell: (cellInfo: {value: ErrorFilterModel }) => {
                    return <ToggleFilterSwitch model={cellInfo.value}/>
                }
            }
        ]
    }, []);

    return (
        <ModelTable
            store={ErrorFilterStore}
            columns={columns}
            filtered={[{id: 'filter_target_id', value: [machineType.getId()]}, {id: 'filter_target_type', value: ['MachineType']}]}
            noLinkColumns={['name','error_codes', 'behaviour', 'enabled']}
            showCreateButton={false}
            suppressWrapper
            suppressActionColumn
        />
    )
});