import * as React from "react";
import { t } from "@lingui/macro"
import {MachineModel} from "../../stores/MachineStore";
import {Filter, ReactTableFunction} from "react-table";
import {SelectField} from "@rx-suite/ui";
import {MachineTypeModel, MachineTypeStore} from "../../stores/MachineTypeStore";

export const useMachineTypeColumn = () => {
    const allMachineTypes = MachineTypeStore.getAll() as MachineTypeModel[];

    const machineType = (type_id: number | null) => {
        if(!type_id){
            return null;
        }
        return allMachineTypes.find(type => type.id == type_id);
    }

    const typeSelectFilter = React.useMemo(()=> {
        const options = [];
        const other = {label: 'other', options: []};
        MachineTypeStore.models.forEach((type) => {
            let category;
            if(type.category && type.category !== 'other'){
                category = options.find(option => option.label == type.category)
                if(!category) {
                    category = {label: type.category, options: []}
                    options.push(category);
                }
            }else {
                category = other;
            }
            category.options.push({label: type.display_name, value: type.id})
        });

        options.sort((a,b) => a.label.localeCompare(b.label))

        // Put 'All' at the top of the list
        options.unshift({label:'All', value: ''});

        // Put 'Other' as last in the list
        options.push(other);

        return options;
    }, [MachineTypeStore.models])

    return (
        {
            Header: t`Machine Type`,
            accessor: (machine: MachineModel) => machine,
            id: 'machine_type_id',
            Cell: (cellInfo: {value: MachineModel }) => {
                const type = machineType(cellInfo.value.machine_type_id);

                if(!type){
                    return null;
                }

                return type.display_name;
            },
            filterable: true,
            Filter: ({filter, onChange}: { filter: Filter, onChange: ReactTableFunction }) => (
                <SelectField
                    options={typeSelectFilter}
                    onChange={(name, value) => onChange(value)}
                    value={filter ? filter.value : ""}
                />
            )
        })
}

