import * as React from 'react';
import {BreadModel, makeColumn} from "@rx-suite/ui";
import {PushTargetModel, PushTargetStore} from "../../../stores/PushTargetStore";
import {ForwardingTargetsInternalForm} from "./ForwardingTargetsInternalForm";
import {t} from "@lingui/macro";
import {Grid} from "@material-ui/core";
import {BreadModelListBadgeEnabled} from "../../../components/generic/bread_model_list_badges/BreadModelListBadgeEnabled";

export const ForwardingTargetsInternalBreadModel = () => {
    const buildEndpoints = React.useCallback((model: PushTargetModel) => {
        const endpoints = [
            {key: 'block', label: 'Blocks'},
            {key: 'calibration', label: 'Calibrations'},
            {key: 'box_coater_event', label: 'EER Events'},
            {key: 'event', label: 'Events'},
            {key: 'error', label: 'Errors'},
            {key: 'job', label: 'Jobs'},
            {key: 'job_state', label: 'Job States'},
            {key: 'machine', label: 'Machines'},
            {key: 'machine_state', label: 'Machine States'},
            {key: 'maintenance', label: 'Maintenances'},
            {key: 'order', label: 'Orders'},
            {key: 'job_process_parameter', label: 'Process Parameters'},
            {key: 'update_order', label: 'Update Orders'}
        ];

        return (
                <Grid container direction={'column'}>
                    {
                        endpoints.map(endpoint => {
                            if(model[endpoint.key] !== null && model[endpoint.key] !== '') {
                                return <BreadModelListBadgeEnabled text={endpoint.label}/>
                            }else{
                                return null;
                            }
                        })
                    }
                </Grid>
            );
    }, []);

    const columns = React.useMemo(() => {
        return [
            makeColumn('app', t`Module Name`, "string"),
            {
                Header: t`Endpoints`,
                accessor: (targets: PushTargetModel) => targets,
                id: 'endpoints',
                Cell: (cellInfo: {value: PushTargetModel }) => {
                    return buildEndpoints(cellInfo.value);
                }
            }
        ]
    }, []);

    return(
        <BreadModel
            store={PushTargetStore}
            Form={ForwardingTargetsInternalForm}
            columns={columns}
        />
    )
};
