import {observer} from "mobx-react-lite";
import {ModelCheckbox, ModelSelect} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import * as React from "react";
import {ProductionSettingStore} from "../../../stores/ProductionSettingStore";
import {LinearProgress} from "@material-ui/core";

export const ProductionMetricsTab = observer(() => {
    const settings = ProductionSettingStore.selectedModel;
    if(!settings) return <LinearProgress />

    const handleToggleSimplifiedMachineStates = () => {
        const proceed = confirm(t`Changing this setting will overwrite the existing machine state colors. Proceed?`)

        if(!proceed){
            return;
        }

        settings.viewModel.simplified_machine_states = !settings.viewModel.simplified_machine_states;
    }

    return(
        <React.Fragment>
            <ModelSelect name={'throughput_metric'}
                         options={
                             [
                                 {label: t`Lens`, value: 'lens'},
                                 {label: t`Pairs`, value: 'pairs'}
                             ]
                         }
                         hint={
                             {
                                 title: t`Throughput Metric`,
                                 text: t`Indicates how jobs are to be counted for throughput calculations.`
                             }
                         }
            />
            <div>
                <ModelCheckbox
                    name={"simplified_machine_states"}
                    onChange={handleToggleSimplifiedMachineStates}
                    hint={
                        {
                            title: t`Simplified Machine States`,
                            text: t`When enabled, groups several machines states for reporting purposes.`
                        }
                    }/>
            </div>
        </React.Fragment>
    )
})