import {attribute, DataStore, model, Model} from '@rx-suite/data';
import {t} from "@lingui/macro";
import * as yup from 'yup';

export type EventDataObj = Readonly<{ event_data: EventData }>;

export type EventData = ReadonlyArray<Readonly<{
  name: string;
  value: string;
}>>;

export type EventDataError = Readonly<{ error: string }>;

@model({
  transportLayer: "/api/events",
  routingInfo: { }
})

export class EventModel extends Model {
  @attribute({ label: t`Message`, validation: yup.string() })
  message: string = "";

  @attribute({ label: t`Type`, validation: yup.number() })
  event_type: number = 0;

  @attribute({ label: t`Time`, validation: yup.date().required() })
  time: Date = new Date();

  @attribute({ label: t`Machine Id`, validation: yup.string().required() })
  machine_sn: string  = "";

  @attribute({ label: t`Created At`, autoForm: false })
  created_at: Date = new Date();

  @attribute({ label: t`Updated At`, autoForm: false })
  updated_at: Date = new Date();

  static types = {
    time: "date"
  }

  async getEventData(): Promise<EventDataObj | EventDataError> {
    const resp = await fetch(`${EventStore.transportLayer.base}/${this.id}/event_data`);

    const respJson = await resp.json();

    if (respJson.error) {
      return { error: respJson.error };
    }

    return { event_data: respJson.event_data };
  }

  constructor() {
    super();
    super.setup();
  }
}

export const EventStore = EventModel.store as DataStore<EventModel>;
