import {attribute, DataStore, model, Model} from "@rx-suite/data";

import {t} from "@lingui/macro";
import * as yup from 'yup';
import {override} from "mobx";
import {JobStore} from "./JobStore";

export type JobStateName = 'started' | 'aborted' | 'finished' | 'postponed' | 'skipped' | 'restarted';

export type ProcessParamsObj = Readonly<{ process_params: ProcessParams }>;

export type ProcessParams = ReadonlyArray<Readonly<{
  name: string;
  value: string;
}>>;

export type ProcessParamsError = Readonly<{ error: string }>;

@model({
  transportLayer: "/api/job_states",
  routingInfo: {}
})
export class JobStateModel extends Model {
  @attribute({label: t`Job`, connects: "JobModel"})
  job_id: string | null = null;

  @attribute({ label: t`Batch Id`, validation: yup.string().nullable()})
  batch_id: string  = "";

  @attribute({ label: t`Process`, validation: yup.string().required() })
  process: string  = "";

  @attribute({ label: t`Side`, validation: yup.string().required() })
  side: string  = "";

  @attribute({ label: t`State`, validation: yup.string().required() })
  state: string  = "";

  @attribute({ label: t`Comment`, validation: yup.string().nullable() })
  comment: string  = "";

  @attribute({ label: t`Scrap`, validation: yup.boolean().nullable() })
  scrap: boolean  = false;

  @attribute({ label: t`Scrap Reason`, validation: yup.boolean().nullable() })
  scrap_reason: string  = "";

  @attribute({ label: t`Time`, validation: yup.date().required() })
  time: Date = new Date();

  @attribute({ label: t`Machine Id`, validation: yup.string().required() })
  machine_id: string  = "";

  @attribute({ label: t`Substate`, validation: yup.string().nullable() })
  substate: string  = "";

  @attribute({ label: t`Created At`, autoForm: false })
  created_at: Date = new Date();

  @attribute({ label: t`Updated At`, autoForm: false })
  updated_at: Date = new Date();

  static types = {
    time: "date"
  }

  @override
  updateFromJson(json: Model, isClone?: boolean) {
    super.updateFromJson(json, isClone);
    let job = JobStore.data.get(json.job.id.toString());
    if(!job){
      job = JobStore.createModel(json.job);
      JobStore.setModel(job)
    }
    this.job_id = job.getId()
  }

  async getProcessParams(): Promise<ProcessParamsObj | ProcessParamsError> {
    const resp = await fetch(`${JobStateStore.transportLayer.base}/${this.id}/process_params`);

    const respJson = await resp.json();

    if (respJson.error) {
      return { error: respJson.error };
    }

    return { process_params: respJson.process_params };
  }

  constructor() {
    super();
    super.setup();
  }
}

export const JobStateStore = JobStateModel.store as DataStore<JobStateModel>;
