class AddBulkJobState < ApplicationJob
  include DataHelpers
  queue_as :high_priority

  def perform(add_params)
    ActiveRecord::Base.connection_pool.with_connection do
      saved_records = create_transaction(add_params[:job_states])
      return if saved_records.nil?
      should_broadcast = add_params[:should_broadcast]
      saved_records.each do |saved_record|
        if should_broadcast
          saved_record.broadcast # broadcast to other Rx-Modules
        end
        saved_record.forward # forward data to other external systems
        KpiJobStateJob.perform_later(saved_record)
      end
    end
  end

  def create_transaction(job_states)
    records = []
    JobState.transaction do
      job_states.map do |job_state|
        record = JobState.create!({
                           job: job_state[:job],
                           machine: job_state[:machine],
                           side: job_state[:side],
                           state: job_state[:state],
                           comment: job_state[:comment],
                           scrap_reason: job_state[:scrap_reason],
                           substate: job_state[:substate],
                           process: job_state[:process],
                           batch_id: nil, # never passed in
                           scrap: nil, # never passed in
                           time: job_state[:time]
                         })
        records << record
      end
    end
    records
  rescue => e
    # in the case of ANY error, return nil, as the record has been rolled back
    nil
  end
end
