class ForwardOrderDataJob < ApplicationJob
  queue_as :machine_data_forwarding

  class FailedToForward < StandardError; end

  def perform(forwarding_path, serialized_params)
    push_targets = MachineDataPushTarget.where.not(forwarding_path => [nil, ""])
    
    push_targets.each do |target|
      endpoint = target[forwarding_path]

      next if endpoint.blank?

      url = "#{target.base_url}/#{endpoint}"
      Rails.logger.info "Forwarding to #{url}. Data => #{serialized_params}"
      response = Faraday.send(:put, url) do |req|
        req.headers['Content-Type'] = 'application/json'
        req.body = serialized_params
      end
      parsed_response = begin
                          Oj.load(response.body, symbol_keys: true)
                        rescue # If json parsing fails, pass through the body
                          response.body
                        end
      raise parsed_response[:error] || parsed_response[:err] || parsed_response.to_s || "Unknown error - consult logs" if response.status == 500
      parsed_response
    rescue => err
      Rails.logger.error "Failed to forward to #{url}. Data => : #{serialized_params}. Error: #{err}"
      raise FailedToForward
    end
  end

  def reschedule_at(current_time, attempts)
    Time.now + 5.minutes
  end

  def max_attempts
    10
  end
end
