class BlockPickingRequest

  attr_accessor :error, :data

  def initialize(external_dcs_id, tray_number, request_tags)
    @tags = nil
    @tray_number = tray_number
    @requested_tags = request_tags

    tags_to_fetch = request_tags.map(&:tag).concat(base_tags)

    resp = RxModule::Communicator.call_module_method_as_module("rx_link", "/api/dcs_configs/#{external_dcs_id}/job_call", :post, {
      job_call: {
        job: @tray_number,
        tags: tags_to_fetch
      }
    })

    return @error = resp[:error] if resp[:error]

    @tags = resp[:tags]

    @data = block_data
  end

  def block_data
    {
      trayNumber: @tray_number,
      do_left: !(@tags[:left].nil?),
      do_right: !(@tags[:right].nil?),
      jobNumber: tag_value_side_less(:JOB),
      diameter: {
        right: diameter(tag_value_for_side(:right, :_BLKDM)),
        left: diameter(tag_value_for_side(:left, :_BLKDM))
      },
      curvature: {
        right: curvature(tag_value_for_side(:right, :_BLKRD)),
        left: curvature(tag_value_for_side(:left, :_BLKRD))
      },
      rework: tag_value_side_less(:_REWORK),
      tagValues: custom_tags
    }
  end

  private

  def base_tags
    %w[_BLKDM _BLKRD _REWORK]
  end

  # Extract the value of a tag when L/R doesn't matter, prioritize left
  def tag_value_side_less(tag)
    return tag_value_for_side(:left, tag) unless tag_value_for_side(:left, tag).nil?
    return tag_value_for_side(:right, tag)  unless tag_value_for_side(:right, tag).nil?
    nil
  end

  def tag_value_for_side(side, tag)
    return @tags[side][tag] unless @tags[side].nil?
    nil
  end

  def value_to_int(raw_val)
    begin
      val_as_float = Float raw_val
      return Integer val_as_float
    rescue
      nil
    end
  end

  def diameter(value)
    return nil if value.blank?
    value_to_int(value)
  end

  def curvature(value)
    return nil if value.blank?
    curvature_type[value_to_int(value)]
  end

  def curvature_type
    {
      60  => 9,
      80  => 7,
      110 => 5,
      135 => 4,
      180 => 3,
      300 => 2,
      500 => 1
    }
  end

  def custom_tags
    @requested_tags.map do |tag|
      {
        tag: tag.name,
        left: format_tag(tag_value_for_side(:left, tag.tag.to_sym) , tag.regex),
        right: format_tag(tag_value_for_side(:right, tag.tag.to_sym) , tag.regex),
      }
    end
  end

  def format_tag(value, regex)
    return 'NO DATA' if value.nil?

    return value if regex.nil?

    value.to_s[/#{regex}/]
  end
end
