module FindHelpers
  module OrderFinders
    class << self
      include NormalizeJobNumber

      # Returns active or expired order that matches number
      def latest_order_with_job_number(job_number)
        all_orders_with_job_number(job_number).first
      end

      # Returns all active or expired order that matches number
      def all_orders_with_job_number(job_number)
        Order.where(job_number: job_number).order("expiry_date DESC")
      end

      def find_active_by_number(order_number)
        Order.active.where(job_number: order_number).order("expiry_date DESC").first
      end

      def find_active_or_create_by_number(order_number)
        order = find_active_by_number(order_number)
        if order.nil?
          order = Order.create(job_number: order_number)
        end
        order
      end
    end
  end
end
