module KpiTableHandler
  module Base
    module UsePlannedProductionTimeCalculator
      def from_sub_query_to_productive_time_sub_query(from_sub_query, start_time, end_time)
        query = from_sub_query.where(ppt_shift_where_intervals(start_time, end_time))

        return query if remove_non_ppt_buckets?

        # we want to zero out buckets not in the shifts
        buckets_within_shifts = query.except(:select).select(:time).except(:group)
        zeroed_out_query = db_handler.from_sub_query(table, zeroed_base_select, base_group)
                                     .where("time not in (?)", buckets_within_shifts)

        zeroed_out_query = attach_wheres(zeroed_out_query, start_time, end_time)

        table.select(Arel.star).from(sub_table_to_from_string(Arel::Nodes::UnionAll.new(query.arel, zeroed_out_query.arel), 'sub'))
      end

      # Returns an array of wheres, one for each shift range interval
      def ppt_shift_where_intervals(interval_start, interval_end)
        # Overlapping merged shifts is already done by hub
        shift_ranges = ShiftCache.get_shift_ranges(interval_start, interval_end)[:ranges]

        #If there's no shifts in the interval, we don't want any records.
        return kpi_table_class.arel_table[:time].eq(Time.at(0)) if shift_ranges.length == 0

        s = shift_ranges.map do |shift_range|
          shift_start = ShiftTimeHelpers.ensure_shift_time_fits_cache(shift_range[:start])
          shift_end = ShiftTimeHelpers.ensure_shift_time_fits_cache(shift_range[:end])
          kpi_table_class.arel_table[:time].gteq(shift_start.to_s).and(kpi_table_class.arel_table[:time].lt(shift_end.to_s))
        end
        # transform into a single where statement
        s.reduce {|s, where| s.or(where)}
      end
    end
  end
end