module KpiTableHandler
  class RoutingKickoutsKpiHandler < KpiTableHandler::Base::KpiTableHandler
    include DataHelpers
    include BucketCalculator

    attr_accessor :reject_reasons, :sum_as

    def initialize(start_time, end_time, timeframe, machine_ids, params: {}, options: {})
      self.reject_reasons = params.key?(:reject_reasons) ? params[:machine_states] : routing_kickout_database_keys_arr
      self.sum_as = params[:sum_as]

      super(start_time, end_time, timeframe, machine_ids, options: options)
    end

    def as_sum?
      !!sum_as
    end

    def aggregate_machines?
      !!options[:aggregate_machines]
    end

    def base_select
      select = routing_kickout_database_keys_arr.map do |db_key|
        KpiRoutingKickout.arel_table[db_key].sum.as(db_key)
      end
      select << KpiRoutingKickout.arel_table[:machine_id] unless aggregate_machines?
      select
    end

    def base_group
      group = []
      group << KpiRoutingKickout.arel_table[:machine_id] unless aggregate_machines?
      group
    end

    def timeframe_select
      selects = []
      if as_sum?
        selects << "SUM(rejects) as rejects"
      else
        selects.concat(routing_kickout_database_keys_arr.map do |db_key|
          KpiRoutingKickout.arel_table[db_key].sum.as(db_key)
        end)
      end
      selects << KpiRoutingKickout.arel_table[:machine_id] unless aggregate_machines?
      selects
    end

    def timeframe_group
      base_group
    end

    def kpi_to_extract
      if as_sum?
        s = "(" + (reject_reasons.each_with_index.map do |db_key, i|
          next "#{db_key}" if i == 0
          " + #{db_key}"
        end.join) + ") as #{sum_as}"
      else
        s = reject_reasons.each_with_index.map do |db_key, i |
          next "#{db_key}" if i == 0
          ", #{db_key}"
        end&.join
      end
      s += ", machine_id" unless aggregate_machines?
      s
    end

    def table
      KpiRoutingKickout
    end

    def col_name
      cols = []
      if as_sum?
        cols << sum_as
      else
        cols.concat routing_kickout_database_keys_arr
      end
      cols << :machine_id unless aggregate_machines?
      cols
    end

    def get_results(query)
      results = {}
      KpiHelpers.get_time_ordered_batches(query) do |batch|
        batch.each do |record|
          time = record[:time]
          if aggregate_machines?
            results[time] = record.as_json.select {|k,v| col_name.map(&:to_sym).include? k.to_sym}
          else
            results[record[:machine_id]] ||= {}
            results[record[:machine_id]][time] = record.as_json.select {|k,v| (col_name.map(&:to_sym).reject {|key| key == :machine_id}).include? k.to_sym}
          end
        end
      end
      results
    end

    private def apply_hash_kpi_operation(hash)
      built_hash = hash
      if as_sum?
        built_hash = hash_sum_values(hash)
      end
      built_hash
    end

    def record_value(record)
      result = {}
      if as_sum?
        result[sum_as.to_sym] = record[sum_as.to_sym]
      else
        reject_reasons.each do |state|
          result[state] = record[state]
        end
      end
      result
    end

    def hash_targets_col
      reject_reasons
    end

    def hash_sum_values(hash)
      return machine_sum_values(hash) if aggregate_machines?
      built_hash = hash
      hash.each do |machine_id, time_buckets|
        built_hash[machine_id] = machine_sum_values(time_buckets)
      end
      built_hash
    end

    def machine_sum_values(hash)
      built_hash = hash
      hash.each do |time, bucket|
        h = {}
        h[sum_as.to_sym] ||= {}
        h[sum_as.to_sym] = (bucket.map {|k,v| v}).reduce(&:+)
        built_hash[time] = h
      end
      built_hash
    end
  end
end
