module MachineKpiCalculator
  class Downtime < MachineKpiCalculator

    include KpiTableHandler

    class << self
      def get_for_period_and_machines(start_time, end_time, timeframe, machine_ids, options)
        kpi = Downtime.new(start_time, end_time, timeframe, machine_ids, options)
        kpi.run
      end
    end

    def initialize(start_time, end_time, timeframe, machine_ids, options)
      super(start_time, end_time, timeframe, machine_ids, options)
    end

    def handlers
      [
        {
          key: :machine_state_duration,
          handler: KpiTableHandler::MachineStateDurationKpiHandler,
          params: {
            machine_states: %w[error halt estop],
            sum_as: 'downtime'
          }
        }
      ]
    end

    # Returns a hash of time buckets and the sum of time in 'downtime' by all specified machines
    # Utilization is expressed in a number of seconds unless 'as_percent' is passed in as an option
    # in which case the value returned is a percentage value.
    # ex:
    # {
    #   {13h00 01/01/2022 : 1600}
    #   {14h00 01/01/2022 : 747}
    # }
    def build_results
      data[:machine_state_duration].each {|k,v| data[:machine_state_duration][k] = v[:downtime]}
    end
  end
end