module MachineKpiCalculator
  class EerKpiCalculator
    KEY_MAPPING = {
      programmed_stop: "Programmed Production Stop",
      breakdown_machine: "Breakdown",
      breakdown_utility: "Breakdown",
      maintenance_usual_pm: "Maintenance",
      maintenance_usual_daily: "Maintenance",
      idle_time_open: "Set-up/Changeover",
      idle_time_wait: "Set-up/Changeover",
      production_setup: "Set-up/Changeover",
      speed_loss: "Speed Loss",
      unknown: "Unknown",
      active_time: "Production Time"
    }

    # Get the events within shifts and calls the block
    def self.timeline_calculator(start_time, end_time, machine_ids, include_in_multiple_shifts = false, &block)
      return if ensure_test_env(start_time, end_time, machine_ids, &block)
      if Rails.env.test?
        machine_ids.each do |machine_id|
          timeline = BoxCoaterEvent.events_for_machines_between(start_time, end_time, machine_id).to_a
          event_indices_within_timeline = 0.step(timeline.length - 1, 1).to_a
          block.call(timeline, event_indices_within_timeline, start_time, end_time)
        end
      end
      shifts_between = RxModule::Communicator::Shifts.shifts_between(start_time, end_time)[:models]
      machine_ids.each do |machine_id|
        # Get all Box Coater events within start and end time
        # Get last box coater before start time
        timeline = BoxCoaterEvent.events_for_machines_between(start_time, end_time, machine_id).to_a
        last_event_before_start = BoxCoaterEvent.last_event_before_time(start_time, machine_id)
        timeline.unshift(last_event_before_start) if last_event_before_start
        index_for_events_already_taken = {}

        if shifts_between.empty?
          # If there's no shift setup, then all time is planned uptime
          event_indices_within_timeline = 0.step(timeline.length - 1, 1).to_a
          block.call(timeline, event_indices_within_timeline, start_time, end_time)
        else
          shifts_between.each do |shift|
            shift[:dates].each do |date|
              shift_start_time = Time.zone.parse(date[:start])
              shift_end_time = Time.zone.parse(date[:end])
              actual_start_time = [start_time, shift_start_time].max
              actual_end_time = [end_time, shift_end_time].min

              event_indices_within_timeline = timeline.each_index.select do |i|
                (
                  (timeline[i].time >= shift_start_time) &&
                  (timeline[i].time < shift_end_time) &&
                  (!index_for_events_already_taken.has_key?(i))
                ) ||
                (include_in_multiple_shifts &&
                  # Events that are longer than a single shift should go back into the timeline for further calculations
                  (timeline[i].time + event_duration(timeline[i], end_time) >= shift_start_time) &&
                  (timeline[i].time < shift_end_time)
                )
              end

              # We should not take the same event to be calculated twice if shifts overlap
              event_indices_within_timeline.each { |i| index_for_events_already_taken[i] = true }
              block.call(timeline, event_indices_within_timeline, actual_start_time, actual_end_time)
            end
          end
        end
      end
    end

    def self.ensure_test_env(start_time, end_time, machine_ids, &block)
      if Rails.env.test?
        machine_ids.each do |machine_id|
          timeline = BoxCoaterEvent.events_for_machines_between(start_time, end_time, machine_id).to_a
          event_indices_within_timeline = 0.step(timeline.length - 1, 1).to_a
          block.call(timeline, event_indices_within_timeline, start_time, end_time)
        end
        return true
      end
      false
    end

    def self.event_duration(event, end_time)
      event.duration || end_time - event.time
    end

    def self.reporting_categories_mapping(bucket)
      new_bucket = {}
      bucket.each do |reporting_category, value|
        new_bucket.merge!({
           KEY_MAPPING[reporting_category.to_sym] => value
         }
        ) { |k, val1, val2| {count: val1[:count] + val2[:count], percent: val1[:percent] + val2[:percent]} }
      end
      new_bucket
    end

    def self.event_duration_in_shift(event, shift_start_time, shift_end_time)
      time = event.time
      duration = EerKpiCalculator.event_duration(event, shift_end_time)

      max_of_start_time = [time, shift_start_time].max
      min_of_end_time = [time + duration, shift_end_time].min

      min_of_end_time - max_of_start_time
    end
  end
end