module MachineKpiCalculator
  class MeanTimeToRestore < MachineKpiCalculator

    include KpiTableHandler
    include MachineErrorHelpers
    include IntervalBuilder

    class << self
      def get_for_period_and_machines(start_time, end_time, timeframe, machine_ids, options: {aggregate_machines: true})
        kpi = MeanTimeToRestore.new(start_time, end_time, timeframe, machine_ids, options)
        kpi.run
      end
    end

    def initialize(start_time, end_time, timeframe, machine_ids, options)
      super(start_time, end_time, timeframe, machine_ids, options)
    end

    def handlers
      [
        {
          key: :machine_state_duration,
          handler: KpiTableHandler::MachineStateDurationKpiHandler,
          params: {
            machine_states: %w[error halt estop],
            sum_as: 'downtime'
          }
        },
        {
          key: :number_of_failures,
          handler: :get_number_of_failures,
          params: {}
        }
      ]
    end

    def get_number_of_failures
      result_builder(start_time, end_time) do |s_time, e_time|
        result = {}
        machine_ids.each do |machine_id|
          machine = Machine.find_by(id: machine_id)
          return 0 if machine.nil?
          result.merge!(machine.number_of_failures_between(s_time, e_time, self.timeframe)) {|k, a_val, b_val| a_val.merge(b_val) {|k, a, b| a + b} }
        end
        result
      end
    end

    # Returns the MTTR, calculated by the formula: MTTR = Total downtime / Total NoF
    # MTTR is expressed as a value in a number of seconds
    # Currently MTTR aggregates all the time buckets to a single value for the period
    def build_results
      summed_downtime = data[:machine_state_duration].values.sum {|k| k[:downtime]}
      summed_failures = data[:number_of_failures].values.sum {|k| k[:failures]}
      (summed_downtime/[summed_failures,1].max).to_f.round(2)
    end

    def format_results
      @results = {count: @results}
    end
  end
end