module AssociateByTimeAndMachine
  extend ActiveSupport::Concern

  # Find a model associated to another model by using when the resource was created (and from which machine)
  def find_associated_by_time_and_machine(resource, use_bounds, extra_where={})
    @use_bound = use_bounds

    after = resource.where(machine_id: machine_id).where(extra_where).where(resource.arel_table[:time].gteq(time)).order(time: :asc).first
    before = resource.where(machine_id: machine_id).where(extra_where).where(resource.arel_table[:time].lteq(time)).order(time: :desc).first

    return nil if after.nil? && before.nil?

    return ensure_time_within_bounds(before, time_association_before_bound) if after.nil?
    return ensure_time_within_bounds(after, time_association_after_bound) if before.nil?

    after_distance = time_distance(after)
    before_distance = time_distance(before)

    if after_distance < before_distance
      return ensure_time_within_bounds(after, time_association_after_bound)
    elsif before_distance < after_distance
      return ensure_time_within_bounds(before, time_association_after_bound)
    else
      return nil # If the distance is equal to both model, the association is ambiguous
    end
  end

  # Overwrite this in your base model to change the time sensitivity of models after
  def time_association_before_bound
    5.minutes
  end

  # Overwrite this in your base model to change the time sensitivity of models before
  def time_association_after_bound
    5.minutes
  end

  private

  def ensure_time_within_bounds(model, bound)
    if @use_bounds
      check_bounds(model, bound)
    else
      return model
    end
  end

  def check_bounds(model, bound)
    if time_distance(model) <= bound
      return model
    else
      return nil
    end
  end

  def time_distance(model)
    return (time - model.time).abs
  end
end
