module ErrorAndAbortPercentage
  extend ActiveSupport::Concern

  def class_name
    self.class.name.split("::")[-1]
  end

  def index_key
    case class_name
    when 'TopErrors', 'ErrorTrends'
      :error_count
    when 'TopAborts', 'AbortTrends'
      :abort_count
    else
      ''
    end
  end

  def trend?
    %w[ErrorTrends AbortTrends].include? class_name
  end

  def total_count_trend
    total_error_count_hash = {}
    # Count add up errors per hour
    @results.each do |error, time_hash|
      time_hash.each do |time, value|
        total_error_count_hash[time] ||= 0
        total_error_count_hash[time] += value[:count]
      end
    end
    total_error_count_hash
  end

  def total_count
    total_error_count_hash = {}
    if aggregate_machines?
      if timeframe_all?
        total_error_count_hash.merge!(@results) {|k, v1, v2| merge_with_addons!(k, v1, v2)}
      else
        @results.each do |time, error_hash|
          total_error_count_hash.merge!(error_hash) {|k, v1, v2| merge_with_addons!(k, v1, v2)}
        end
      end
    else
      @results.each do |machine_id, buckets|
        if timeframe_all?
          total_error_count_hash.merge!(buckets) {|k, v1, v2| merge_with_addons!(k, v1, v2)}
        else
          buckets.each do |time, error_hash|
            total_error_count_hash.merge!(error_hash) {|k, v1, v2| merge_with_addons!(k, v1, v2)}
          end
        end
      end
    end
    total_error_count_hash.values.map {|x| x[:count]}.sum.to_f
  end

  def timeframe_all_percent_bucket(row)
    t = total_count
    row.each do |ck, cv|
      row[ck][:percent] = Tools.percentage_maker(cv[:count], t, 0)
    end
  end

  def percent_buckets(row, m_count)
    return timeframe_all_percent_bucket(row) if timeframe == KpiHelpers::Timeframe::ALL

    if trend?
      t = total_count_trend
      row.each do |error, time_hash|
        time_hash.each do |ck, cv|
          row[error][ck][:percent] = Tools.percentage_maker(cv[:count], t[ck], 0)
        end
      end
    else
      raise NotImplementedError
    end
  end
end