# == Schema Information
#
# Table name: job_breakages
#
#  id                   :bigint           not null, primary key
#  accepted             :boolean          default(FALSE)
#  job_number           :string(4000)
#  posted_at            :datetime
#  posted_by            :string(4000)
#  validated_at         :datetime
#  validated_by         :string(4000)
#  posted_by_user_id    :integer
#  validated_by_user_id :integer
#
# Indexes
#
#  index_job_breakages_on_job_number  (job_number)
#
class JobBreakage < ApplicationRecord
  has_many :job_breakage_data

  def resolve_breakage_data
    {
      left: job_breakage_data.joins(:job).where(job: {side: 0}).first,
      right: job_breakage_data.joins(:job).where(job: {side: 1}).first
    }
  end

  def should_broadcast?
    false
  end

  def validate_breakage(accepted, validated_by, validated_by_id)
    update(accepted: accepted,
           validated_by: validated_by,
           validated_by_user_id: validated_by_id,
           validated_at: Time.zone.now
    )
  end

  def is_validated?
    !validated_by.nil?
  end

  def self.delete_before(date)
    max_delete_queue = 5
    return 0 if max_delete_queue.nil? || max_delete_queue <= 0
    deleted_objects = 0
    # Delete the calibrations that have existed before the given date
    # Only delete a maximum of `max_delete_queue` calibrations
    ActiveRecord::Base.connected_to(shard: :primary, role: :writing) do
      job_breakages = JobBreakage.where(JobBreakage.arel_table[:posted_at].lt(date)).limit(max_delete_queue)
      deleted_objects += job_breakages.delete_all
    end
    deleted_objects
  end
end
