module MachineTypes
    class MachineTypeTemplate
      class << self
        def seed
          machine_type = get_machine_type

          return puts "Machine Type #{display_name} (version #{machine_type.seed_version}) up to date - skipping seed." unless machine_type.seed_version != seed_version

          puts "Updating Machine Type #{display_name} (version #{machine_type.seed_version}) to seed version #{seed_version}"

          error_filters.each do |filter_def|
            resolved_group_code = "#{name}_#{filter_def[:group_code]}"

            if ErrorFilter.find_by(group_code: resolved_group_code).nil?
              machine_type.error_filters.build({
                   group_code: resolved_group_code,
                   name: filter_def[:name],
                   error_codes: filter_def[:error_codes],
                   system_default: true,
                   behaviour: filter_def[:behaviour],
                   replace_name: filter_def[:replace_name]
                })
            end
          end

          abort_filters.each do |filter_def|
            resolved_group_code = "#{name}_#{filter_def[:group_code]}"

            if AbortFilter.find_by(group_code: resolved_group_code).nil?
              machine_type.abort_filters.build({
                                                 group_code: resolved_group_code,
                                                 name: filter_def[:name],
                                                 regexps: filter_def[:regexps],
                                                 system_default: true,
                                                 behaviour: filter_def[:behaviour],
                                                 replace_name: filter_def[:replace_name]
                                               })
            end
          end

          apply_patch

          machine_type.seed_version = seed_version

          machine_type.save!
          puts "Machine Type #{display_name} updated to seed version #{seed_version}"
        end

        def seed_version
          raise NotImplementedError
        end

        def apply_patch
          # Overwrite this in the machine type class to provide update to the machine type data
        end

        def get_machine_type
          machine_type = MachineType.find_by_name(name)
          if machine_type.nil?
            machine_type = MachineType.create(
              name: name,
              display_name: display_name,
              seed_version: 0,
              system_default: true,
              category: category,
              image_url: image_name.nil? ? nil : "#{category}/#{image_name}",
              machine_state_enabled: interfaces[:machine_state],
              job_state_enabled: interfaces[:job_state],
              error_enabled: interfaces[:error],
              maintenance_enabled: interfaces[:maintenance],
              process_data_enabled: interfaces[:process_data],
              event_data_enabled: interfaces[:event],
              calibration_data_enabled: interfaces[:calibration],
              translator_class: translator_class,
            )
          end
          machine_type
        end

        def display_name
          raise NotImplementedError
        end

        def name
          raise NotImplementedError
        end

        def error_filters
          []
        end

        def abort_filters
          []
        end

        def translator_class
          nil
        end
      end
    end
end

