module ReportPeriod
  extend ActiveSupport::Concern
  include KpiHelpers

  class_methods do
    def use_report_range
      parameter(
        :report_range,
        "Timeframe",
        "What will be the timeframe of the buckets",
        :date_picker,
        default: {
          timeframe: 'hour by hour',
          report_start: nil,
          report_end: nil
        },
        required: true)
    end

    def use_planned_production_time
      parameter(
        :use_planned_production_time,
        "Use Planned Production Time",
        "Will skip time that is not within a shift as unproductive time.",
        :boolean
      )
    end
  end

  def self.parse_report_range(range_params)
    report_start = Time.zone.parse(range_params[:report_start])
    timeframe_key = range_params[:timeframe].downcase.to_sym
    timeframe = KpiHelpers::Timeframe::TIMEFRAME_TO_ENUM[timeframe_key]
    report_end = Time.zone.parse(range_params[:report_end])

    {
      report_start: report_start,
      report_end: report_end,
      timeframe: timeframe
    }
  end

  def report_start
    @parameters[:report_range][:report_start]
  end

  def report_end
    @parameters[:report_range][:report_end]
  end

  def timeframe
    @parameters[:report_range][:timeframe]
  end

  def planned_production_time
    !!@parameters[:use_planned_production_time]
  end
end