class EerReport < MachineTemplateReport
  include ReportPeriod

  class << self
    def report_type
      "EER Report"
    end

    def report_enabled?
      ProductionSetting.use_eer_metrics? # Disable this report if we are not tracking EER
    end

    def eer_reporting_target
      options = []
      group_option = {label: 'Groups', options: []}
      box_coater_options = {
        label: 'Box Coaters',
        options: []
      }
      group_option[:options] = MachineGroup.eer_machine_groups.map {|machine_group| { label: target_to_name(machine_group), value: target_to_id(machine_group) }}
      options << group_option
      box_coater_options[:options] = Machine.where(track_eer_metrics: true).map do |machine|
        { label: machine.extended_name, value: target_to_id(machine) }
      end
      options << box_coater_options
      options
    end
  end

  expose('eer_report', report_type, "EER Report for Box Coaters.", "EERReport")
  parameter(
    :target_ids,
    "Box Coaters",
    "The box coaters that will be the target of the report",
    :string,
    multiple: true,
    required: true,
    value_list: -> {eer_reporting_target})

  def components
    [
      {
        component: ReportComponents::BoxCoaterEerBreakdown,
        args: {colorize_series: true}
      },
      {
        component: ReportComponents::BoxCoaterTimeBreakdownPie,
        args: {colorize_series: true, event_types: [:engineering_lockout]}
      },
      {
        component: ReportComponents::BoxCoaterTimeInReportingCategoryBreakdownPie,
        args: {}
      },
      {
        component: ReportComponents::BoxCoaterYieldLine,
        args: {colorize_series: true}
      },
      {
        component: ReportComponents::BoxCoaterBatchLoadingBar,
        args: {colorize_series: true}
      }
    ]
  end
end
