class MachineTemplateReport < RxModule::Reporting::Report
  include KpiHelpers
  include ReportMachines
  include ReportComponents
  include ReportPeriod

  ui_hint(category: 'Machine Report')

  use_report_range #parameters[:timeframe]
  use_machine_targets(true) #paramerts[:machine_ids]

  def report_type
    raise NotImplementedError
  end

  def initialize(parameter_hash)
    # Remove the report range from the base param as the report engine doesn't know what to do with it
    super(parameter_hash.except(:report_range))
    @parameters[:report_range] = ReportPeriod.parse_report_range(parameter_hash[:report_range])
  end

  def run
    {
      meta: {
        version: '3',
        resolved_targets: resolve_machine_targets,
        report_type: self.class.report_type,
        report_start: report_start,
        report_end: report_end
      },
      components: execute_components
    }
  end

  def components
    raise NotImplementedError
  end

  def execute_components
    components.map do |c| c[:component].new(report_start,
                                            report_end,
                                            resolve_machine_ids,
                                            timeframe,
                                            planned_production_time)
                                       .build(c[:args])
    end
  end
end