module ReportComponents
  class BoxCoaterEerBreakdown < ReportComponent
     include ReportComponentHelpers
     include BucketCalculator
     include KpiHelpers

    def component_id
      'box_coater_eer_breakdown'
    end

    def component_type
      'line'
    end

    def generate(args)
      @timeframe = Timeframe::DAY_BY_DAY
      yield_ = MachineKpi.box_coater_yield_between(start_time, end_time, @timeframe, machine_ids, options: args)
      batch_ = MachineKpi.box_coater_batch_loading_between(start_time, end_time, @timeframe, machine_ids, options: args)
      time_in_events_ = MachineKpi.time_in_event_type_breakdown(start_time, end_time, @timeframe, machine_ids, options: args)
      time_in_reporting_category_ = MachineKpi.time_in_reporting_category_breakdown(start_time, end_time, @timeframe, machine_ids, options: args)

      # We calculate the average over the timeframe and return that
      # For EER value, we calculate a daily eer and then return the average of that
      # Eer formula: time performance * loading performance * quality performance
      total_yield = 0
      total_batch = 0
      total_fully_productive_time = 0
      timeframe_eer = []
      buckets = time_buckets(start_time, end_time, @timeframe)
      buckets.each do |time, value|
        total_yield += yield_[time][:percent]
        total_batch += batch_[time]
        planned_prod_time = time_in_events_[time][:active_time][:count] - time_in_events_[time][:engineering_lockout][:count]
        fully_productive_time = planned_prod_time - time_in_reporting_category_[time].except(:active_time).values.sum {|h| h[:count]}
        fully_prod_time_over_active_time = Tools.percentage_maker(fully_productive_time, time_in_events_[time][:active_time][:count]).round(2)
        total_fully_productive_time += fully_prod_time_over_active_time
        timeframe_eer << ((yield_[time][:percent] / 100) * (batch_[time] / 100) * (fully_prod_time_over_active_time / 100))
      end

      avg_yield = (total_yield / buckets.size).round(2)
      avg_batch = (total_batch / buckets.size).round(2)
      avg_fully_prod_time = (total_fully_productive_time / buckets.size).round(2)
      avg_eer = ((timeframe_eer.sum / buckets.size) * 100).round(2)

      {
        yield: {
          label: @timeframe > Timeframe::DAY_BY_DAY ? "avg Yield" : "Yield",
          value: "#{avg_yield}%"
        },
        batch: {
          label: @timeframe > Timeframe::DAY_BY_DAY ? "avg Loading" : "Loading",
          value: "#{avg_batch}%"
        },
        productive_time: {
          label: @timeframe > Timeframe::DAY_BY_DAY ? "avg Fully Productive Time" : "Fully Productive Time",
          value: "#{avg_fully_prod_time}%"
        },
        eer_value: {
          label: @timeframe > Timeframe::DAY_BY_DAY ? "avg EER Value" : "EER Value",
          value: avg_eer
        }
      }
    end
  end
end
